package org.jtheque.core.managers.core;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.state.AbstractState;
import org.jtheque.utils.bean.IntDate;

/**
 * The configuration of the core.
 *
 * @author Baptiste Wicht
 */
public final class CoreConfiguration extends AbstractState {
    private static final String TRUE = "true";
    private static final String FALSE = "false";
    
    /**
     * Cette mthode retourne le look and feel prfr de l'utilisateur.
     *
     * @return le L&f prfr
     */
    public String getPreferedLookAndFeel() {
        return getProperty("lookAndFeel");
    }

    /**
     * Cette mthode modifie le look and feel prfr de l'utilisateur.
     *
     * @param lookAndFeel le L&f prfr
     */
    public void setPreferedLookAndFeel(String lookAndFeel) {
        setProperty("lookAndFeel", lookAndFeel);
    }

    /**
     * Cette mthode permet de savoir si on vrifie les mises  jour au dmarrage.
     *
     * @return <code>true</code> s'il faut vrifier les mises  jour au dmarrage sinon <code>false</code>
     */
    public boolean verifiingUpdateOnStartup() {
        return Boolean.parseBoolean(getProperty("verifyOnStartup", FALSE));
    }

    /**
     * Cette mthode indique si le systme doit vrifier les mises  jour au dmarrage.
     *
     * @param value true si on veut vrifier les mises  jour au dmarrage
     */
    public void setVerifiingUpdateOnStartup(boolean value) {
        setProperty("verifyOnStartup", String.valueOf(value));
    }

    /**
     * Cette mthode permet de savoir s'il faut retenir ou non la taille et la position de la fentre pour la
     * restaurer  l'ouverture de l'application.
     *
     * @return <code>true</code> ou <code>false</code>
     */
    public boolean retainSizeAndPositionOfWindow() {
        return Boolean.parseBoolean(getProperty("retainSizeAndPositionOfWindow", FALSE));
    }

    /**
     * Cette mthode permet d'indiquer s'il faut retenir ou non la taille et la position de la fentre pour la
     * restaurer  l'ouverture de l'application.
     *
     * @param value Valeur indiquant si on doit retenir la taille et la position de la fentre
     */
    public void setRetainSizeAndPositionOfWindow(boolean value) {
        setProperty("retainSizeAndPositionOfWindow", Boolean.toString(value));
    }

    /**
     * Cette mthode indique s'il faut toujours appliquer ce look and feel.
     *
     * @return <code>true</code> ou <code>false</code>
     */
    public boolean alwaysLookAndFeel() {
        return Boolean.parseBoolean(getProperty("alwaysLookAndFeel", TRUE));
    }

    /**
     * Cette mthode permet d'indiquer s'il faut toujours appliquer ce look and feel.
     *
     * @param value Valeur indiquant si on doit toujours appliquer ce L&F
     */
    public void setAlwaysLookAndFeel(boolean value) {
        setProperty("alwaysLookAndFeel", Boolean.toString(value));
    }

    /**
     * Cette mthode indique s'il y a un proxy ou non.
     *
     * @return <code>true</code> ou <code>false</code>
     */
    public boolean hasAProxy() {
        return Boolean.parseBoolean(getProperty("proxy", FALSE));
    }

    /**
     * Cette mthode permet d'indiquer s'il y a un proxy.
     *
     * @param value Valeur indiquant s'il y a un proxy ou non
     */
    public void setHasAProxy(boolean value) {
        setProperty("proxy", Boolean.toString(value));
    }

    /**
     * Cette mthode permet d'indiquer le port du proxy.
     *
     * @param value Port du proxy
     */
    public void setProxyPort(String value) {
        setProperty("proxyPort", value);
    }

    /**
     * cette mthode permet de rcuprer le port du proxy.
     *
     * @return Le port
     */
    public String getProxyPort() {
        return getProperty("proxyPort");
    }

    /**
     * Cette mthode permet d'indiquer l'addresse du proxy.
     *
     * @param value Adresse du proxy
     */
    public void setProxyAddress(String value) {
        setProperty("proxyAddress", value);
    }

    /**
     * cette mthode permet de rcuprer l'adresse du proxy.
     *
     * @return L'adresse
     */
    public String getProxyAddress() {
        return getProperty("proxyAddress");
    }

    /**
     * Cette mthode permet d'indiquer s'il faut supprimmer ou non les logs.
     *
     * @param value <code>true</code> s'il faut supprimmer les logs <code>else</code>
     */
    public void setMustDeleteLogs(boolean value) {
        setProperty("deleteLog", Boolean.toString(value));
    }

    /**
     * cette mthode permet de savoir s'il faut ou non supprimmer les logs.
     *
     * @return <code>true</code> s'il faut les supprimmer, sinon <code>false</code>
     */
    public boolean mustDeleteLogs() {
        return Boolean.valueOf(getProperty("deleteLog", FALSE));
    }

    /**
     * Cette mthode permet d'indiquer aprs combien de jours, il faut supprimmer les logs.
     *
     * @param value Le nombre de jours
     */
    public void setNumberOfDaysBeforeDeleteLogs(int value) {
        setProperty("daysBeforeDeleteLogs", Integer.toString(value));
    }

    /**
     * cette mthode retourne le nombre de jours aprs lequel il faut supprimmer les logs.
     *
     * @return Le nombre de jours
     */
    public int getNumberOfDaysBeforeDeleteLogs() {
        return Integer.parseInt(getProperty("daysBeforeDeleteLogs", "0"));
    }

    /**
     * Set the email of the user.
     *
     * @param value The new email to set
     */
    public void setUserEmail(String value) {
        setProperty("userEmail", value);
    }

    /**
     * Return the email of the user.
     *
     * @return The email
     */
    public String getUserEmail() {
        return getProperty("userEmail");
    }

    /**
     * Set the SMTP host.
     *
     * @param value The new SMTP host.
     */
    public void setSmtpHost(String value) {
        setProperty("smtpHost", value);
    }

    /**
     * Return the SMTP host.
     *
     * @return The SMTP host.
     */
    public String getSmtpHost() {
        return getProperty("smtpHost");
    }

    /**
     * Set the date of the last reading of the messages.
     *
     * @param value The date of the last reading of the messages.
     */
    void setMessagesLastRead(String value) {
        setProperty("messagesLastRead", value);
    }

    /**
     * return the date of the last reading of the messages.
     *
     * @return The date of the last reading of the messages.
     */
    public IntDate getMessagesLastRead() {
        if (getProperty("messagesLastRead") == null) {
            setMessagesLastRead("20000101");
        }

        return new IntDate(Integer.parseInt(getProperty("messagesLastRead")));
    }

    /**
     * Set the configuration to the default values.
     */
    public void setDefaults() {
        setAlwaysLookAndFeel(true);
        setHasAProxy(false);
        setMustDeleteLogs(false);
        setNumberOfDaysBeforeDeleteLogs(5);
        setPreferedLookAndFeel("Metal");
        setProxyAddress("");
        setProxyPort("");
        setRetainSizeAndPositionOfWindow(true);
        setSmtpHost("");
        setUserEmail("");
        setMessagesLastRead("20000101");
        setVerifiingUpdateOnStartup(false);
    }
}