package org.jtheque.core.managers.module;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.AbstractManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.managers.module.beans.ModuleState;
import org.jtheque.core.managers.module.beans.CollectionBasedModule;
import org.jtheque.core.managers.module.beans.ModuleContainer;
import org.jtheque.core.managers.module.beans.EmptyBeanMethod;
import org.jtheque.core.managers.module.loaders.IModuleLoader;
import org.jtheque.core.managers.module.loaders.ModuleLoader;
import org.jtheque.core.managers.module.annotations.Module;
import org.jtheque.core.managers.module.annotations.ModuleDefinition;
import org.jtheque.core.managers.module.annotations.PrimaryModule;
import org.jtheque.core.managers.update.InstallationResult;
import org.jtheque.core.managers.update.IUpdateManager;
import org.jtheque.core.managers.update.repository.Repository;
import org.jtheque.core.managers.update.repository.RepositoryReader;
import org.jtheque.core.managers.update.repository.ModuleDescription;
import org.jtheque.core.managers.view.IViewManager;

import org.jtheque.utils.io.FileUtils;
import org.jtheque.utils.collections.CollectionUtils;
import org.jtheque.utils.bean.Version;

import javax.annotation.Resource;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Collection;

/**
 * A module manager implementation. It manage the cycle life of the modules.
 *
 * @author Baptiste Wicht
 */
public final class ModuleManager extends AbstractManager implements IModuleManager {
    /**
     * The application repository.
     */
    private Repository repository;

    /**
     * The current primary module.
     */
    private ModuleContainer primaryModule;

    /**
     * The configuration of the module manager. It seems the informations about the modules who're
     * installed or disabled.
     */
    private ModuleConfiguration configuration;

    @Resource
    private IModuleLoader moduleLoader;

    private final List<ModuleContainer> moduleContainers = new ArrayList<ModuleContainer>(10);
    private final Collection<ModuleContainer> modulesToLoad = new ArrayList<ModuleContainer>(10);

    /**
     * Indicate if we must refresh the list of the modules to load.
     */
    private boolean mustRefresh;

    @Override
    public void preInit(){
        moduleContainers.addAll(ModuleLoader.getModules());

        loadModuleBeans();

        for(JThequeError error : ModuleLoader.getErrors()){
            Managers.getManager(IViewManager.class).displayError(error);
        }
    }

    /**
     * Load the module beans.  
     */
    private void loadModuleBeans() {
        for (ModuleContainer module : moduleContainers) {
            module.setModule(Managers.getManager(IBeansManager.class).getBean(module.getBeanName()));
        }
    }

    @Override
    public void init() {
        //Nothing to init
    }

    @Override
    public void close(){
        for (ModuleContainer module : moduleContainers) {
            if (module.getState() == ModuleState.UNINSTALLED) {
                configuration.remove(module);

                //We create a "tag" (a file) to say to the application to delete it at the next launch
                FileUtils.createEmptyFile(
                        module.getModuleFile().getAbsolutePath().replace(
                                module.getModuleFile().getName(),
                                "deleted" + module.getModuleFile().getName()));
            }
        }
    }

    /**
     * Preplug the modules.
     */
    @Override
    public void prePlugModules() {
        for (ModuleContainer module : getModulesToLoad()) {
            module.getPrePlugMethod().run();
            setState(module, ModuleState.LOADED);
        }
    }

    /**
     * Plug the modules.
     */
    @Override
    public void plugModules() {
        for (ModuleContainer module : getModulesToLoad()) {
            module.getPlugMethod().run();
        }
    }

    /**
     * Return the modules to load.
     *
     * @return A List containing all the modules to load.
     */
    private Collection<ModuleContainer> getModulesToLoad() {
        if (mustRefresh) {
            modulesToLoad.clear();

            modulesToLoad.add(primaryModule);

            addLoadableModules();

            mustRefresh = false;
        }

        return modulesToLoad;
    }

    /**
     * Add the loadable modules to the modules to load list. 
     */
    private void addLoadableModules() {
        for (ModuleContainer module : moduleContainers) {
            if (canBeLoaded(module)) {
                String dependency = getDependency(module);

                if (dependency == null || dependency.isEmpty()) {
                    modulesToLoad.add(module);
                } else if (dependency.equals(primaryModule.getInfos().name())) {
                    modulesToLoad.add(module);
                }
            }
        }
    }

    /**
     * Test if the module can be loaded. 
     * 
     * @param module The module to test. 
     * 
     * @return true if the module can be loaded else false. 
     */
    private static boolean canBeLoaded(ModuleContainer module) {
        return !module.isPrimaryModule() && !(module.getState() == ModuleState.DISABLED || module.getState() == ModuleState.UNINSTALLED);
    }

    /**
     * Return the dependency of the module. 
     * 
     * @param module The module to get the dependency from. 
     * 
     * @return The dependency of the module or null if the module has no dependency. 
     */
    private static String getDependency(ModuleContainer module) {
        if (module.hasModuleDefinition()) {
            return module.getModuleDefinition().dependency();
        }
        
        return null;
    }

    /**
     * Unplug the modules.
     */
    @Override
    public void unplugModules() {
        List<ModuleContainer> modulesToUnplug = CollectionUtils.copyOf(getModulesToLoad());

        CollectionUtils.reverse(modulesToUnplug);

        for (ModuleContainer module : modulesToUnplug) {
            if (module.getState() == ModuleState.LOADED) {
                module.getUnPlugMethod().run();

                setState(module, ModuleState.INSTALLED);
            }
        }
    }

    /**
     * Configure the modules.
     */
    private void configureModules() {
        configuration = getStates().getState(ModuleConfiguration.class);

        if (configuration == null) {
            initConfiguration();
        }

        CollectionUtils.filter(moduleContainers, new ConfigurationFilter(configuration));
        CollectionUtils.filter(moduleContainers, new CoreVersionFilter());

        CollectionUtils.sort(moduleContainers, new ModuleComparator());
    }

    /**
     * Init the configuration of the modules.  
     */
    private void initConfiguration() {
        try {
            configuration = getStates().createState(ModuleConfiguration.class);
        } catch (Exception e) {
            configuration = new ModuleConfiguration();
            getLogger().exception(e);
            getErrors().addInternationalizedError("error.loading.configuration");
        }

        for (ModuleContainer module : moduleContainers) {
            configuration.add(module, ModuleState.INSTALLED);
        }
    }

    @Override
    public Collection<ModuleContainer> getModules() {
        return moduleContainers;
    }

    @Override
    public List<ModuleContainer> getPrimaryModules() {
        List<ModuleContainer> primaryModules = new ArrayList<ModuleContainer>(5);

        for (ModuleContainer module : moduleContainers) {
            if (module.isPrimaryModule()) {
                primaryModules.add(module);
            }
        }

        return primaryModules;
    }

    @Override
    public List<ModuleDescription> getModulesFromRepository() {
        return getRepository().getModules();
    }

    @Override
    public Repository getRepository() {
        if (repository == null) {
            repository = new RepositoryReader().read(Managers.getCore().getApplication().getRepository());
        }

        return repository;
    }

    /**
     * Set the primary module.
     *
     * @param primaryModule The primary module.
     */
    @Override
    public void setPrimaryModule(ModuleContainer primaryModule) {
        if (!primaryModule.isPrimaryModule()) {
            throw new IllegalArgumentException("The module must be a primary module");
        }

        mustRefresh = true;

        this.primaryModule = primaryModule;
    }

    @Override
    public ModuleContainer getPrimaryModule() {
        return primaryModule;
    }

    /**
     * Set the state of a module.
     *
     * @param module The module to set the state.
     * @param state  The state.
     */
    void setState(ModuleContainer module, ModuleState state) {
        module.setState(state);
        configuration.setState(module.getInfos().name(), state);
    }

    /**
     * Load a module.
     *
     * @param module The module to load.
     */
    @Override
    public void loadModule(ModuleContainer module) {
        setState(module, ModuleState.LOADED);

        module.getPrePlugMethod().run();
        module.getPlugMethod().run();
    }

    @Override
    public void enableModule(ModuleContainer module) {
        setState(module, ModuleState.INSTALLED);
    }

    /**
     * Disable a module.
     *
     * @param module The module to disable.
     */
    @Override
    public void disableModule(ModuleContainer module) {
        setState(module, ModuleState.DISABLED);

        module.getUnPlugMethod().run();

        mustRefresh = true;
    }

    /**
     * Install a module.
     *
     * @param file The file of the module.
     *
     * @return true if the module has been installed, else false.
     */
    @Override
    public boolean installModule(File file) {
        Object module = moduleLoader.installModule(file);

        if(module == null){
            return false;
        }

        ModuleContainer container = createSimpleContainer(module);

        moduleContainers.add(container);

        configuration.add(container);

        fireModuleAdded();

        return true;
    }

    @Override
    public void install(String versionsFileURL) {
        InstallationResult result = Managers.getManager(IUpdateManager.class).install(versionsFileURL);

        if (result.isInstalled() && result.isMustRestart()) {
            configuration.add(result);

            Managers.getManager(IViewManager.class).displayI18nText("message.module.repository.installed.restart");
        } else if (result.isInstalled()) {
            Object module = moduleLoader.loadModule(new File(
                    Managers.getCore().getFolders().getModulesFolder(), result.getJarFile()));

            ModuleContainer container = createSimpleContainer(module);

            moduleContainers.add(container);

            configuration.add(result);

            fireModuleAdded();

            getManager(IViewManager.class).displayI18nText("message.module.repository.installed");
        } else {
            getManager(IViewManager.class).displayI18nText("error.repository.module.not.installed");
        }
    }

    /**
     * Create a simple module container for the module. 
     * 
     * @param module The module. 
     * 
     * @return The module container for the specified module. 
     */
    private static ModuleContainer createSimpleContainer(Object module) {
        ModuleContainer container = new ModuleContainer("notABean", 
                module.getClass().getAnnotation(Module.class), 
                module.getClass().getAnnotation(ModuleDefinition.class), 
                module.getClass().getAnnotation(PrimaryModule.class) != null);

        container.setModule(module);
        container.setState(ModuleState.JUST_INSTALLED);
        container.setPlugMethod(new EmptyBeanMethod());
        container.setPrePlugMethod(new EmptyBeanMethod());
        container.setUnPlugMethod(new EmptyBeanMethod());

        return container;
    }

    @Override
    public void choosePrimaryModule() {
        configureModules();

        if (getPrimaryModules().isEmpty()){
            getManager(IViewManager.class).displayI18nText("error.module.primary.none");
            Managers.getCore().getLifeCycleManager().exit();
        } else if (getPrimaryModules().size() > 1) {
            getManager(IViewManager.class).displayPrimaryModuleList();
        } else {
            setPrimaryModule(getPrimaryModules().get(0));

            Managers.getCore().getLifeCycleManager().launchNextPhase();
        }
    }

    /**
     * Uninstall a module.
     *
     * @param module The module to uninstall.
     */
    @Override
    public void uninstallModule(ModuleContainer module) {
        setState(module, ModuleState.UNINSTALLED);

        module.getUnPlugMethod().run();
        
        fireModuleRemoved(module);
    }

    @Override
    public void addModuleListener(ModuleListener listener) {
        getListeners().add(ModuleListener.class, listener);
    }

    @Override
    public void removeModuleListener(ModuleListener listener) {
        getListeners().remove(ModuleListener.class, listener);
    }

    /**
     * Fire a module added event.
     */
    private static void fireModuleAdded() {
        ModuleListener[] l = getListeners().getListeners(ModuleListener.class);

        for (ModuleListener listener : l) {
            listener.moduleAdded();
        }
    }

    /**
     * Fire a module removed event.
     *
     * @param module The removed module.
     */
    private static void fireModuleRemoved(ModuleContainer module) {
        ModuleListener[] l = getListeners().getListeners(ModuleListener.class);

        for (ModuleListener listener : l) {
            listener.moduleRemoved(module);
        }
    }
    
    @Override
    public String canModuleLaunched(ModuleContainer module) {
        if (module.isPrimaryModule()) {
            return getMessage("error.module.not.loaded.primary");
        } else if (new Version(module.getInfos().coreVersion()).isGreaterThan(Managers.getCore().getCoreCurrentVersion())) {
            return getMessage("modules.message.versionproblem");
        } else if (areNotAllDependenciesSatisfied(module)) {
            return getMessage("error.module.not.loaded.dependency", module.getModuleDefinition().dependency());
        } else if (module.getState() == ModuleState.JUST_INSTALLED){
            return getMessage("error.module.just.installed");
        }

        return "";
    }

    /**
     * Indicate if not all the dependencies of the modules are not satisfied. 
     * 
     * @param module The module to test. 
     * 
     * @return true if not all the dependencies of the module are not satisfied else false. 
     */
    private boolean areNotAllDependenciesSatisfied(ModuleContainer module){
        if (module.hasModuleDefinition() && module.getModuleDefinition().dependency() != null &&
                    !module.getModuleDefinition().dependency().isEmpty()) {
            ModuleContainer dependency = getModule(module.getModuleDefinition().dependency());

            return dependency == null || dependency.getState() != ModuleState.LOADED;
        }

        return false;
    }

    @Override
    public ModuleContainer getModule(String name) {
        ModuleContainer module = null;

        for (ModuleContainer m : moduleContainers) {
            if (name.equals(m.getInfos().name())) {
                module = m;
                break;
            }
        }

        return module;
    }

    @Override
    public boolean isInstalled(String module) {
        return getModule(module) != null;
    }

    /**
     * Launch the primary module.
     */
    @Override
    public void launchPrimaryModule() {
        mustRefresh = true;

        getManager(IViewManager.class).displaySplashScreen();
        
        prePlugModules();
        plugModules();

        getManager(IViewManager.class).getViews().getMainView().display();

        getManager(IViewManager.class).closeSplashScreen();
    }

    /**
     * Swith the primary module.
     */
    @Override
    public void switchPrimaryModule() {
        unplugModules();
        
        getManager(IViewManager.class).getViews().getMainView().refresh();
        getManager(IViewManager.class).getViews().getModuleView().closeDown();
        getManager(IViewManager.class).getViews().getMainView().closeDown();

        Managers.getManager(IViewManager.class).displayPrimaryModuleList();
    }

    @Override
    public boolean isCollectionModule() {
        return CollectionBasedModule.class.isAssignableFrom(primaryModule.getModule().getClass());
    }

    @Override
    public boolean chooseCollection(String collection, String password, boolean create) {
        return ((CollectionBasedModule)primaryModule.getModule()).chooseCollection(collection, password, create);
    }

    @Override
    public void plugCollection() {
        ((CollectionBasedModule)primaryModule.getModule()).plugCollection();
    }
}