package org.jtheque.core.managers.view.impl.components;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdesktop.jxlayer.JXLayer;
import org.jdesktop.jxlayer.plaf.BufferedLayerUI;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

/**
 * A tabbedpane with different layer.
 *
 * @author Baptiste Wicht
 */
public class LayerTabbedPane extends JTabbedPane {
    private static final long serialVersionUID = -5183713814998385959L;

    private final List<JXLayer<JComponent>> components;

    /**
     * Construct a new LayerTabbedPane.
     */
    public LayerTabbedPane() {
        super();

        components = new ArrayList<JXLayer<JComponent>>(5);

        addChangeListener(new TabbedAnimatingChangeListener());
    }

    /**
     * Add an internationalized tab.
     *
     * @param key       The internationalization key.
     * @param component The component to add.
     */
    public final void addInternationalizedTab(String key, JComponent component) {
        addTab(Managers.getManager(ILanguageManager.class).getMessage(key), component);
    }

    /**
     * Add a tab.
     *
     * @param title     The title of the tab.
     * @param component The tab component.
     */
    public final void addTab(String title, JComponent component) {
        JXLayer<JComponent> layerComponent = new JXLayer<JComponent>(component);

        components.add(layerComponent);
        super.addTab(title, layerComponent);
    }

    /**
     * Return the selected component.
     *
     * @return the selected component.
     */
    @Override
    public final JComponent getSelectedComponent() {
        if (getSelectedIndex() != -1) {
            return components.get(getSelectedIndex()).getView();
        }

        return null;
    }

    /**
     * Return the selected layer.
     *
     * @return the selected layer or null if there is no selected layer.
     */
    public final JXLayer<JComponent> getSelectedLayer() {
        JXLayer<JComponent> layer = null;

        if (getSelectedIndex() != -1) {
            layer = components.get(getSelectedIndex());
        }

        return layer;
    }

    /**
     * Return the layer at a certain index.
     *
     * @param index The index for which we search the layer.
     * @return the corresponding layer.
     */
    public final JXLayer<JComponent> getLayerAt(int index) {
        return components.get(index);
    }

    static class TabbedAnimatingChangeListener implements ChangeListener {
        private int index;
        private Timer timer;
        private AnimationLayerUI layerUI;
        private float delta;

        public TabbedAnimatingChangeListener() {
            this(50, .1f);
        }

        public TabbedAnimatingChangeListener(int delay, final float delta) {
            this.delta = delta;
            layerUI = new AnimationLayerUI();

            timer = new Timer(delay, new ActionListener() {
                public void actionPerformed(ActionEvent e) {

                    if (layerUI.getAlpha() <= delta) {
                        layerUI.setAlpha(0);
                        timer.stop();
                        return;
                    }

                    layerUI.setAlpha(layerUI.getAlpha() - delta);
                }
            });
        }

        public float getDelta() {
            return delta;
        }

        public void setDelta(float delta) {
            if (delta <= 0 || delta > 1) {
                throw new IllegalArgumentException();
            }
            this.delta = delta;
        }

        public int getDelay() {
            return timer.getDelay();
        }

        public void setDelay(int delay) {
            timer.setDelay(delay);
        }

        @SuppressWarnings("unchecked")
        public void stateChanged(ChangeEvent e) {
            LayerTabbedPane pane = (LayerTabbedPane) e.getSource();
            JXLayer<JComponent> layer = pane.getSelectedLayer();
            JXLayer<JComponent> oldLayer = pane.getLayerAt(index);

            if(oldLayer != null && layer != null){
                layerUI.setAlpha(1 - layerUI.getAlpha());
                
                layerUI.setComponent(oldLayer);
                
                oldLayer.setUI(layer.getUI());
                layer.setUI(layerUI);
    
                //painter.update();            
                timer.start();
                index = pane.getSelectedIndex();
            }
        }
    }

    static class AnimationLayerUI extends BufferedLayerUI<JComponent> {
        private BufferedImage componentImage;

        public AnimationLayerUI() {
            setAlpha(0);
            setIncrementalUpdate(false);
        }

        public void setComponent(JComponent component) {
            if (component == null
                    || component.getWidth() <= 0 || component.getHeight() <= 0) {
                componentImage = null;
            } else {
                if (componentImage == null
                        || componentImage.getWidth() != component.getWidth()
                        || componentImage.getHeight() != component.getHeight()) {
                    componentImage = createBuffer(component.getWidth(), component.getHeight());
                }
                Graphics g = componentImage.getGraphics();
                component.paint(g);
                g.dispose();
            }
        }

        public void paint(Graphics g, JComponent c) {
            // always paint the layer
            c.paint(g);
            super.paint(g, c);
        }

        protected void paintLayer(Graphics2D g2, JXLayer<JComponent> l) {
            if (componentImage != null) {
                // paint the old layer with diminishing alpha
                g2.drawImage(componentImage, 0, 0, null);
            }
        }
    }
}