package org.jtheque.core.managers.view.impl.frame;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.skin.SkinListener;
import org.jtheque.core.managers.skin.ISkinManager;
import org.jtheque.core.managers.lifecycle.listeners.TitleEvent;
import org.jtheque.core.managers.lifecycle.listeners.TitleListener;
import org.jtheque.core.managers.view.MainController;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.view.able.IMainView;
import org.jtheque.core.managers.view.impl.components.JThequeStateBar;
import org.jtheque.core.managers.view.impl.components.LayerTabbedPane;
import org.jtheque.core.managers.view.impl.components.ExtendedGlassPane;
import org.jtheque.core.managers.view.impl.components.WaitFigure;
import org.jtheque.core.managers.view.impl.components.InfiniteWaitFigure;
import org.jtheque.core.managers.view.impl.components.menu.JMenuBarJTheque;
import org.jtheque.core.managers.view.impl.frame.abstraction.SwingFrameView;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.core.utils.ui.Borders;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.utils.ui.GridBagUtils;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.annotation.Resource;
import javax.annotation.PostConstruct;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * The main view of JTheque.
 *
 * @author Baptiste Wicht
 */
public final class MainView extends SwingFrameView implements TitleListener, IMainView, SkinListener {
    private static final long serialVersionUID = -5931960632421815283L;

    private final Component glassPane = new ExtendedGlassPane(this);
    private final WaitFigure waitFigure = new InfiniteWaitFigure();

    private LayerTabbedPane tab;

    private MainController controller;

    private int defaultWidth;
    private int defaultHeight;

    private WindowListener tempListener;

    @Resource
    private IViewManager viewManager;

    @Override
    @PostConstruct
    public void build() {
        setTitle(Managers.getCore().getLifeCycleManager().getTitle());
        setResizable(false);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        tempListener = new TempWindowAdapter();

        addWindowListener(tempListener);

        Container background = new JPanel();
        background.setBackground(viewManager.getViewDefaults().getFilthyBackgroundColor());
        setContentPane(background);

        setSize(defaultWidth, defaultHeight);
        setMinimumSize(new Dimension(defaultWidth, defaultHeight));

        SwingUtils.centerFrame(this);
    }

    @Override
    public void fill() {
        setIconImage(getDefaultWindowIcon());
        setResizable(true);
        controller = new MainController();

        setContentPane(buildContentPane());
        setJMenuBar(new JMenuBarJTheque());

        removeWindowListener(tempListener);

        addWindowListener(controller);

        configureGlassPane();

        Managers.getCore().getLifeCycleManager().addTitleListener(this);
        Managers.getManager(ISkinManager.class).addSkinListener(this);

        viewManager.configureView(this, "main", defaultWidth, defaultHeight);
    }

    @Override
    public void configureGlassPane() {
        setGlassPane(glassPane);
        glassPane.setVisible(true);
        setWaitFigure(waitFigure);
    }

    /**
     * Cette mthode initialise et retourne le contentPane (le JTabbedPane).
     *
     * @return le contentPane
     */
    private Container buildContentPane() {
        PanelBuilder builder = new PanelBuilder();
        builder.getPanel().setBorder(Borders.EMPTY_BORDER);

        if (viewManager.isTabMainComponent()) {
            if(Managers.getManager(IViewManager.class).getTabComponents().isEmpty()){
                Component emptyPanel = new JPanel();
                emptyPanel.setBackground(viewManager.getViewDefaults().getBackgroundColor());

                viewManager.setMainComponent(emptyPanel);

                builder.add(emptyPanel, builder.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 1.0, 1.0));
            } else {
                tab = new MainTabbedPane();
                tab.addChangeListener(controller);

                builder.add(tab, builder.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 1.0, 1.0));
            }
        } else {
            builder.add((Component) viewManager.getMainComponent(), builder.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 1.0, 1.0));
        }

        builder.add(new JThequeStateBar(), builder.gbcSet(0, 1, GridBagUtils.HORIZONTAL, GridBagUtils.LAST_LINE_START));

        return builder.getPanel();
    }

    @Override
    public LayerTabbedPane getTabbedPane() {
        return tab;
    }

    @Override
    public void titleUpdated(TitleEvent event) {
        setTitle(event.getTitle());
    }

    @Override
    public void closeDown() {
        viewManager.saveState(this, "main");

        super.closeDown();
    }

    @Override
    public boolean validateContent() {
        return true;
    }

    @Override
    public void setSelectedComponent(Object component) {
        tab.setSelectedComponent((Component)component);
    }

    @Override
    public JComponent getSelectedComponent() {
        return tab.getSelectedComponent();
    }

    /**
     * Set the default width of the mainView. This is not for use, only for Spring injection. 
     * 
     * @param defaultWidth The default width. 
     */
    public void setDefaultWidth(int defaultWidth) {
        this.defaultWidth = defaultWidth;
    }

    /**
     * Set the default height of the view. This is not for use, only for Spring injection. 
     * 
     * @param defaultHeight The default height. 
     */
    public void setDefaultHeight(int defaultHeight) {
        this.defaultHeight = defaultHeight;
    }

    @Override
    public void skinChanged() {
        refresh();
    }

    /**
     * The temporaray window adapter. 
     * 
     * @author Baptiste Wicht
     */
    private static class TempWindowAdapter extends WindowAdapter {
        @Override
            public void windowClosing(WindowEvent e) {
            Managers.getCore().getLifeCycleManager().exit();
        }
    }
}